/**
 * FlpTime(Flappy Time)
 * (c) Kenta Cho (cs8k-cyu@asahi-net.or.jp)
 * http://www.asahi-net.or.jp/~cs8k-cyu/
 *
 * clockmanager.c(manage clock status)
 */

int ptpTime;

/**
 * initialize hand
 */
static void initHandStatus(HandStatus* hd) {
  hd->fmvTime = hd->gwTime = hd->pwTime = 1;
  hd->gMv = 0;
  hd->deg = hd->amDeg;
  hd->ptPower = 0;
}

/**
 * set current time(hour/minute/second)
 */
static void setTime() {
  DateTimeType dateTime;
  TimSecondsToDateTime(TimGetSeconds(), &dateTime);
  handStatus[0].amDeg = dateTime.hour*5*4 + dateTime.minute/3;
  if ( handStatus[0].amDeg >= 240 ) handStatus[0].amDeg -= 240;
  handStatus[1].amDeg = dateTime.minute*4 + dateTime.second/15;
  handStatus[2].amDeg = dateTime.second*4;
}

/**
 * set current second
 */
static void setSecond() {
  DateTimeType dateTime;
  TimSecondsToDateTime(TimGetSeconds(), &dateTime);
  handStatus[2].amDeg = dateTime.second*4;
}

/**
 * initialize all hands
 */
static void initClock() {
  int i;
  createScreenBuffer();
  setTime();
  for ( i=0 ; i<3 ; i++ ) {
    initHandStatus(&handStatus[i]);
  }
  ptpTime = randN(400)+400;
}

/**
 * try to show accurate time
 */
static void setPtPower() {
  int i;
  setTime();
  for ( i=0 ; i<3 ; i++ ) {
    handStatus[i].ptPower = randN(100)+20;
    handStatus[i].pwTime = -1;
  }
  ptpTime = randN(400)+400;
}

static void screenTaped() {
  /*int i;
  for ( i=0 ; i<3 ; i++ ) {
    if ( handStatus[i].ptPower > 0 ) return;
  }*/
  setPtPower();
}

/*
 * hand handler
 */
static void recalcTime(HandStatus* hd) {
  hd->ptPower--;
  if ( hd->ptPower == 0 ) {
    hd->fmvTime = hd->gwTime = hd->pwTime = 1;
  }
  hd->fmvTime--; 
  if ( hd->fmvTime == 0 ) {
    hd->fmvTime = -9999 + (randN(240)+180);
  } else if ( hd->fmvTime < -9999 ) {
    hd->fmvTime = 5 + randN(15);
    if ( randN(2) ) {
      hd->fMv = randN(8)+2;
    } else {
      hd->fMv = -randN(8)-2;
    }
  }
  hd->gwTime--; 
  if ( hd->gwTime == 0 ) {
    hd->gwTime = -9999 + (randN(400)+80);
  } else if ( hd->gwTime < -9999 ) {
    hd->gwTime = 8;
    if ( randN(2) ) {
      hd->gWidth = 10 + randN(20);
    } else {
      hd->gWidth = -10 - randN(20);
    }
  }
  hd->pwTime--; 
  if ( hd->pwTime == 0 ) {
    hd->pwTime = -9999 + (randN(240)+60);
  } else if ( hd->pwTime < -9999 ) {
    hd->pwTime = randN(10)*2;
    if ( randN(2) ) {
      hd->pWidth = 2 + randN(2);
    } else {
      hd->pWidth = -2 - randN(2);
    }
  }
}

static void moveHand(HandStatus* hd) {
  Boolean gvl = TRUE;
  if ( hd->ptPower > 0 ) {
    int mv;
    gvl = FALSE;
    mv = hd->amDeg - hd->deg;
    if ( mv > 120 ) mv -= 240;
    else if ( mv <= -120 ) mv += 240;
    if ( mv > 0 ) {
      if ( mv > hd->ptPower ) mv = hd->ptPower;
      mv /= 10; if ( mv == 0 ) mv = 1;
    } else if ( mv < 0 ) {
      if ( mv < -hd->ptPower ) mv = -hd->ptPower;
      mv /= 10; if ( mv == 0 ) mv = -1;
    }
    hd->deg += mv;
    if ( hd->ptPower < 20 ) {
      hd->pwTime = hd->ptPower;
      hd->pWidth = 3-(hd->ptPower/7);
    }
  } else {
    if ( hd->fmvTime > 0 ) {
      gvl = FALSE;
      hd->deg += hd->fMv;
    }
    if ( hd->gwTime > 0 ) {
      gvl = FALSE;
      if ( hd->gwTime < 3 ) {
	hd->deg -= hd->gWidth/2;
      } else {
	hd->deg -= hd->gWidth/6;
      }
    }
  }
  if ( hd->pwTime > 0 ) {
    if ( (hd->pwTime&1) ) {
      hd->deg += hd->pWidth;
    } else {
      hd->deg -= hd->pWidth;
    }
  }
  if ( gvl ) {
    hd->gMv += isin(hd->deg)*2;
    if ( hd->deg>60 && hd->deg<180 ) hd->gMv -= hd->gMv/10;
    hd->deg += (hd->gMv/128);
  } else {
    hd->gMv = 0;
  }
  while ( hd->deg < 0 ) hd->deg += 240;
  while ( hd->deg >= 240 ) hd->deg -= 240;
}

/**
 * handle all hands
 */
static void handleClockStatus() {
  int i;
  flipDisplay();
  drawClock();
  for ( i=0 ; i<3 ; i++ ) {
    recalcTime(&handStatus[i]);
    moveHand(&handStatus[i]);
  }
  if ( (handStatus[2].ptPower%10) == 9 ) {
    setSecond();
  } else if ( (handStatus[2].ptPower%10) == -1 ) {
    handStatus[2].deg += 4;
    if ( handStatus[2].deg >= 240 ) handStatus[2].deg -= 240;
  }
  ptpTime--;
  if ( ptpTime <= 0 ) {
    setPtPower();
  }  
}
