/**
 * FlpTime(Flappy Time)
 * (c) Kenta Cho (cs8k-cyu@asahi-net.or.jp)
 * http://www.asahi-net.or.jp/~cs8k-cyu/
 *
 * flptime.c(main routine)
 */

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include "flptime.h"

static HandStatus handStatus[3];

#include "sincos.c"
#include "drawclock.c"
#include "clockmanager.c"

static int StartApplication(void) {
  randomize();
  initClock();
  FrmGotoForm(FLPTIME_FORM);
}

static void StopApplication(void) {
}

/**
 * event hanlder for main form(flptime form)
 */
static Boolean MyFormHandleEvent(EventPtr event) {
  Boolean handled = FALSE;
  switch ( event->eType ) {
  case penDownEvent:
    screenTaped();
    handled = TRUE;
    break;
  case frmOpenEvent:
    FrmDrawForm(FrmGetActiveForm());
    handled = TRUE;
    break;
  }
  return handled;
}

/**
 * activate forms
 */
static Boolean ApplicationHandleEvent(EventPtr event) {
  FormPtr frm;
  Int formID;
  Boolean handled = FALSE;

  if ( event->eType == frmLoadEvent ) {
    formID = event->data.frmLoad.formID;
    frm = FrmInitForm(formID);
    FrmSetActiveForm(frm);
    switch ( formID ) {
    case FLPTIME_FORM:
      FrmSetEventHandler(frm, MyFormHandleEvent);
      break;
    }
    handled = TRUE;
  }
  return handled;
}

// event loop interval
#define INTERVAL 10

static void EventLoop(void) {
  EventType event;
  Word error;
  long ntc, tc;
  int itv;
  ntc = TimGetTicks() + INTERVAL;
  do {
    itv = ntc - TimGetTicks();
    if ( itv < 1 ) itv = 1;
    EvtGetEvent(&event, itv);
    if ( !SysHandleEvent(&event) ) {
      if ( !MenuHandleEvent(0, &event, &error) ) {
	if ( !ApplicationHandleEvent(&event) ) {
	  FrmDispatchEvent(&event);
	}
      }
    }
    tc = TimGetTicks();
    if ( tc >= ntc ) {
      handleClockStatus();
      //ntc = tc + INTERVAL;
      ntc = ((tc+ntc)>>1) + INTERVAL;
    }
  } while ( event.eType != appStopEvent );
}

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags) {
  if ( cmd != sysAppLaunchCmdNormalLaunch ) return 0;
  StartApplication();
  EventLoop();
  StopApplication();
  return 0;
}
